@@ -1,5 +1,5 @@
 package Pegex::Grammar;
-$Pegex::Grammar::VERSION = '0.25';
+$Pegex::Grammar::VERSION = '0.30';
 use Pegex::Base;
 
 # Grammar can be in text or tree form. Tree will be compiled from text.
@@ -13,6 +13,7 @@ has tree => (
     builder => 'make_tree',
     lazy => 1,
 );
+has start_rules => [];
 
 sub make_text {
     my ($self) = @_;
@@ -29,7 +30,10 @@ sub make_tree {
         or die "Can't create a '" . ref($self) .
             "' grammar. No tree or text or file.";
     require Pegex::Compiler;
-    return Pegex::Compiler->new->compile($text)->tree;
+    return Pegex::Compiler->new->compile(
+        $text,
+        @{$self->start_rules || []}
+    )->tree;
 }
 
 # This import is to support: perl -MPegex::Grammar::Module=compile
@@ -71,10 +75,19 @@ sub compile_into_module {
     $module =~ s!::!/!g;
     $module = "$module.pm";
     my $file = $INC{$module} or return;
-#     require Pegex::Bootstrap;
-#     my $perl = Pegex::Bootstrap->new->compile($grammar_text)->to_perl;
-    require Pegex::Compiler;
-    my $perl = Pegex::Compiler->new->compile($grammar_text)->to_perl;
+    my $perl;
+    my @rules;
+    if ($package->can('start_rules')) {
+        @rules = @{$package->start_rules || []};
+    }
+    if ($module eq 'Pegex/Pegex/Grammar.pm') {
+        require Pegex::Bootstrap;
+        $perl = Pegex::Bootstrap->new->compile($grammar_text, @rules)->to_perl;
+    }
+    else {
+        require Pegex::Compiler;
+        $perl = Pegex::Compiler->new->compile($grammar_text, @rules)->to_perl;
+    }
     open IN, $file or die $!;
     my $module_text = do {local $/; <IN>};
     close IN;
